@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2016 Daniel Scharrer
+ * Copyright (C) 2012-2018 Daniel Scharrer
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the author(s) be held liable for any damages
@@ -48,12 +48,12 @@ struct is_unsafe_path_char {
 			case '|': return true;
 			case '?': return true;
 			case '*': return true;
+			default:  return false;
 		}
-		return false;
 	}
 };
 
-static std::string replace_unsafe_chars(const std::string & str) {
+std::string replace_unsafe_chars(const std::string & str) {
 	std::string result;
 	result.resize(str.size());
 	std::replace_copy_if(str.begin(), str.end(), result.begin(), is_unsafe_path_char(), '$');
@@ -160,20 +160,20 @@ std::string filename_map::shorten_path(const std::string & path) const {
 	return result;
 }
 
-std::string filename_map::convert(std::string input) const {
+std::string filename_map::convert(std::string path) const {
 	
 	// Convert paths to lower-case if requested
 	if(lowercase) {
-		std::transform(input.begin(), input.end(), input.begin(), ::tolower);
+		std::transform(path.begin(), path.end(), path.begin(), ::tolower);
 	}
 	
 	// Don't expand variables if requested
 	if(!expand) {
-		return input;
+		return path;
 	}
 	
-	it begin = input.begin();
-	std::string expanded = expand_variables(begin, input.end());
+	it begin = path.begin();
+	std::string expanded = expand_variables(begin, path.end());
 	
 	return shorten_path(expanded);
 }
