@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2015 Daniel Scharrer
+ * Copyright (C) 2012-2016 Daniel Scharrer
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the author(s) be held liable for any damages
@@ -44,7 +44,7 @@ static const char path_sep = '/';
  */
 class filename_map : public std::map<std::string, std::string> {
 	
-	const std::string & lookup(const std::string & key) const;
+	std::string lookup(const std::string & key) const;
 	
 	bool lowercase;
 	bool expand;
