@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2015 Daniel Scharrer
+ * Copyright (C) 2011-2018 Daniel Scharrer
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the author(s) be held liable for any damages
@@ -25,6 +25,8 @@
 #include <limits>
 
 #include <boost/cstdint.hpp>
+#include <boost/algorithm/string/predicate.hpp>
+#include <boost/filesystem/operations.hpp>
 #include <boost/range/size.hpp>
 
 #include "util/console.hpp"
@@ -44,9 +46,8 @@ const char slice_ids[][8] = {
 
 slice_reader::slice_reader(std::istream * istream, boost::uint32_t data_offset)
 	: data_offset(data_offset),
-	  dir(), last_dir(), base_file(), slices_per_disk(1),
-	  current_slice(0), slice_file(), slice_size(0),
-	  ifs(), is(istream) {
+	  slices_per_disk(1), current_slice(0), slice_size(0),
+	  is(istream) {
 	
 	std::streampos max_size = std::streampos(std::numeric_limits<boost::int32_t>::max());
 	
@@ -58,12 +59,12 @@ slice_reader::slice_reader(std::istream * istream, boost::uint32_t data_offset)
 	}
 }
 
-slice_reader::slice_reader(const path_type & dir, const std::string & base_file,
-                           size_t slices_per_disk)
+slice_reader::slice_reader(const path_type & dir, const std::string & basename,
+                           const std::string & basename2, size_t slices_per_disk)
 	: data_offset(0),
-	  dir(dir), last_dir(dir), base_file(base_file), slices_per_disk(slices_per_disk),
-	  current_slice(0), slice_file(), slice_size(0),
-	  ifs(), is(&ifs) { }
+	  dir(dir), base_file(basename), base_file2(basename2),
+	  slices_per_disk(slices_per_disk), current_slice(0), slice_size(0),
+	  is(&ifs) { }
 
 void slice_reader::seek(size_t slice) {
 	
@@ -80,6 +81,10 @@ void slice_reader::seek(size_t slice) {
 
 bool slice_reader::open_file(const path_type & file) {
 	
+	if(!boost::filesystem::exists(file)) {
+		return false;
+	}
+	
 	log_info << "Opening \"" << color::cyan << file.string() << color::reset << '"';
 	
 	ifs.close();
@@ -126,10 +131,6 @@ bool slice_reader::open_file(const path_type & file) {
 		throw slice_error(oss.str());
 	}
 	
-	slice_file = file;
-	
-	last_dir = file.parent_path();
-	
 	return true;
 }
 
@@ -156,6 +157,19 @@ std::string slice_reader::slice_filename(const std::string & basename, size_t sl
 	return oss.str();
 }
 
+bool slice_reader::open_file_case_insensitive(const path_type & dir, const path_type & filename) {
+	
+	boost::filesystem::directory_iterator end;
+	for(boost::filesystem::directory_iterator i(dir); i != end; ++i) {
+		path_type actual_filename = i->path().filename();
+		if(boost::iequals(actual_filename.string(), filename.string()) && open_file(dir / actual_filename)) {
+			return true;
+		}
+	}
+	
+	return false;
+}
+
 void slice_reader::open(size_t slice) {
 	
 	current_slice = slice;
@@ -163,17 +177,28 @@ void slice_reader::open(size_t slice) {
 	ifs.close();
 	
 	path_type slice_file = slice_filename(base_file, slice, slices_per_disk);
-	
-	if(open_file(last_dir / slice_file)) {
+	if(open_file(dir / slice_file)) {
 		return;
 	}
 	
-	if(dir != last_dir && open_file(dir / slice_file)) {
+	path_type slice_file2 = slice_filename(base_file2, slice, slices_per_disk);
+	if(!base_file2.empty() && slice_file2 != slice_file && open_file(dir / slice_file2)) {
+		return;
+	}
+	
+	if(open_file_case_insensitive(dir, slice_file)) {
+		return;
+	}
+	
+	if(!base_file2.empty() && slice_file2 != slice_file && open_file_case_insensitive(dir, slice_file2)) {
 		return;
 	}
 	
 	std::ostringstream oss;
 	oss << "could not open slice " << slice << ": " << slice_file;
+	if(!base_file2.empty() && slice_file2 != slice_file) {
+		oss << " or " << slice_file2;
+	}
 	throw slice_error(oss.str());
 }
 
