@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2015 Daniel Scharrer
+ * Copyright (C) 2011-2018 Daniel Scharrer
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the author(s) be held liable for any damages
@@ -71,6 +71,7 @@
 #include <boost/foreach.hpp>
 #include <boost/static_assert.hpp>
 #include <boost/unordered_map.hpp>
+#include <boost/range/size.hpp>
 
 #include "util/log.hpp"
 #include "util/math.hpp"
@@ -87,235 +88,8 @@ enum known_codepages {
 
 namespace {
 
-static const char replacement_char = '_';
-
-typedef boost::uint32_t unicode_char;
-
-static size_t get_encoding_size(codepage_id codepage) {
-	switch(codepage) {
-		case  1200: return 2u; // UTF-16LE
-		case  1201: return 2u; // UTF-16BE
-		case 12000: return 4u; // UTF-32LE
-		case 12001: return 4u; // UTF-32BE
-		default:    return 1u;
-	}
-}
-
-//! Fallback conversion that will at least work for ASCII characters
-static void to_utf8_fallback(const std::string & from, std::string & to, codepage_id cp) {
-	
-	size_t skip = get_encoding_size(cp);
-	
-	size_t shift = 0;
-	switch(cp) {
-		case  1201: shift = 1u * 8u; break; // UTF-16BE
-		case 12001: shift = 3u * 8u; break; // UTF-32BE
-	}
-	
-	to.clear();
-	to.reserve(ceildiv(from.size(), skip));
-	
-	bool warn = false;
-	
-	for(std::string::const_iterator it = from.begin(); it != from.end();) {
-		
-		unicode_char unicode = 0;
-		for(size_t i = 0; i < skip; i++) {
-			unicode |= unicode_char(boost::uint8_t(*it++)) << (i * 8);
-		}
-		
-		char ascii = char((unicode >> shift) & 0x7f);
-		
-		// replace non-ASCII characters with underscores
-		if((unicode_char(ascii) << shift) != unicode) {
-			warn = true;
-			ascii = replacement_char;
-		}
-		
-		to.push_back(ascii);
-	}
-	
-	if(warn) {
-		static bool warned = false;
-		log_warning << "Unknown data while converting from CP" << cp << " to UTF-8.";
-		if(!warned && (cp == cp_windows1252 || cp == cp_utf16le)) {
-			#if INNOEXTRACT_HAVE_ICONV
-			log_warning << " └─ make sure your iconv installation supports Windows-1252 and UTF-16LE";
-			#elif !INNOEXTRACT_HAVE_BUILTIN_CONV && !INNOEXTRACT_HAVE_WIN32_CONV
-			log_warning << " └─ build innoextract with charset conversion routines enabled!";
-			#endif
-			warned = true;
-		}
-	}
-	
-}
-
-#if INNOEXTRACT_HAVE_BUILTIN_CONV
-
-static size_t utf8_length(unicode_char chr) {
-	if     (chr <  0x80)       return 1;
-	else if(chr <  0x800)      return 2;
-	else if(chr <  0x10000)    return 3;
-	else if(chr <= 0x0010ffff) return 4;
-	return 1;
-}
-
-static void utf8_write(std::string & to, unicode_char chr) {
-	
-	static const boost::uint8_t first_bytes[7] = {
-		0x00, 0x00, 0xc0, 0xe0, 0xf0, 0xf8, 0xfc
-	};
-	
-	// Get number of bytes to write
-	size_t length = utf8_length(chr);
-	
-	// Extract bytes to write
-	boost::uint8_t bytes[4];
-	switch(length) {
-		case 4: bytes[3] = static_cast<boost::uint8_t>((chr | 0x80) & 0xBF), chr >>= 6;
-		case 3: bytes[2] = static_cast<boost::uint8_t>((chr | 0x80) & 0xBF), chr >>= 6;
-		case 2: bytes[1] = static_cast<boost::uint8_t>((chr | 0x80) & 0xBF), chr >>= 6;
-		case 1: bytes[0] = static_cast<boost::uint8_t>(chr | first_bytes[length]);
-	}
-	
-	// Add them to the output
-	const boost::uint8_t * cur_byte = bytes;
-	switch(length) {
-		case 4: to.push_back(char(*cur_byte++));
-		case 3: to.push_back(char(*cur_byte++));
-		case 2: to.push_back(char(*cur_byte++));
-		case 1: to.push_back(char(*cur_byte++));
-	}
-	
-}
-
-//! \return true c is is the first part of an UTF-16 surrogate pair
-static bool is_utf16_high_surrogate(unicode_char chr) {
-	return chr >= 0xd800 && chr <= 0xdbff;
-}
-
-//! \return true c is is the second part of an UTF-16 surrogate pair
-static bool is_utf16_low_surrogate(unicode_char chr) {
-	return chr >= 0xdc00 && chr <= 0xdfff;
-}
-
-static void utf16le_to_utf8(const std::string & from, std::string & to) {
-	
-	if(from.size() % 2 != 0) {
-		log_warning << "Unexpected trailing byte in UTF-16 string.";
-	}
-	
-	to.clear();
-	to.reserve(from.size() / 2); // optimistically, most strings only have ASCII characters
-	
-	bool warn = false;
-	
-	std::string::const_iterator it = from.begin();
-	std::string::const_iterator end = from.end();
-	while(it != end) {
-		
-		unicode_char chr = boost::uint8_t(*it++);
-		if(it == end) {
-			warn = true;
-			utf8_write(to, replacement_char);
-			break;
-		}
-		chr |= unicode_char(boost::uint8_t(*it++)) << 8;
-		
-		// If it's a surrogate pair, convert to a single UTF-32 character
-		if(is_utf16_high_surrogate(chr)) {
-			if(it == end) {
-				warn = true;
-				utf8_write(to, replacement_char);
-				break;
-			}
-			unicode_char d = boost::uint8_t(*it++);
-			if(it == end) {
-				warn = true;
-				utf8_write(to, replacement_char);
-				break;
-			}
-			d |= unicode_char(boost::uint8_t(*it++)) << 8;
-			if(is_utf16_low_surrogate(d)) {
-				chr = ((chr - 0xd800) << 10) + (d - 0xdc00) + 0x0010000;
-			} else {
-				warn = true;
-				utf8_write(to, replacement_char);
-				continue;
-			}
-		}
-		
-		// Replace invalid characters
-		if(chr > 0x0010FFFF) {
-			warn = true;
-			// Invalid character (greater than the maximum unicode value)
-			utf8_write(to, replacement_char);
-			continue;
-		}
-		
-		utf8_write(to, chr);
-	}
-	
-	if(warn) {
-		log_warning << "Unexpected data while converting from UTF-16LE to UTF-8.";
-	}
-	
-}
-
-static void windows1252_to_utf8(const std::string & from, std::string & to) {
-	
-	static unicode_char replacements[] = {
-		0x20ac, replacement_char, 0x201a, 0x192, 0x201e, 0x2026, 0x2020, 0x2021, 0x2c6,
-		0x2030, 0x160, 0x2039, 0x152, replacement_char, 0x17d, replacement_char,
-		replacement_char, 0x2018, 0x2019, 0x201c, 0x201d, 0x2022, 0x2013, 0x2014, 0x2dc,
-		0x2122, 0x161, 0x203a, 0x153, replacement_char, 0x17e, 0x178
-	};
-
-	BOOST_STATIC_ASSERT(sizeof(replacements) == (160 - 128) * sizeof(*replacements));
-	
-	to.clear();
-	to.reserve(from.size()); // optimistically, most strings only have ASCII characters
-	
-	bool warn = false;
-	
-	BOOST_FOREACH(char c, from) {
-		
-		// Windows-1252 maps almost directly to Unicode - yay!
-		unicode_char chr = boost::uint8_t(c);
-		if(chr >= 128 && chr < 160) {
-			chr = replacements[chr - 128];
-			warn = warn || (chr == unicode_char(replacement_char));
-		}
-		
-		utf8_write(to, chr);
-	}
-	
-	if(warn) {
-		log_warning << "Unexpected data while converting from Windows-1252 to UTF-8.";
-	}
-	
-}
-
-static bool to_utf8_builtin(const std::string & from, std::string & to, codepage_id cp) {
-	
-	switch(cp) {
-		case cp_utf16le:     utf16le_to_utf8(from, to); return true;
-		case cp_windows1252: windows1252_to_utf8(from, to); return true;
-		case cp_iso_8859_1:  windows1252_to_utf8(from, to); return true;
-		default:             return false;
-	}
-	
-}
-
-#endif // INNOEXTRACT_HAVE_BUILTIN_CONV
-
-#if INNOEXTRACT_HAVE_ICONV
-
-typedef boost::unordered_map<codepage_id, iconv_t> converter_map;
-static converter_map converters;
-
 //! Get names for encodings where iconv doesn't have the codepage alias
-static const char * get_encoding_name(codepage_id codepage) {
+const char * get_encoding_name(codepage_id codepage) {
 	switch(codepage) {
 		case   708: return "ISO-8859-6";
 		case   936: return "GBK";
@@ -387,7 +161,340 @@ static const char * get_encoding_name(codepage_id codepage) {
 	}
 }
 
-static iconv_t get_converter(codepage_id codepage) {
+typedef boost::uint32_t unicode_char;
+
+const unicode_char replacement_char = '_';
+
+size_t get_encoding_size(codepage_id codepage) {
+	switch(codepage) {
+		case  1200: return 2u; // UTF-16LE
+		case  1201: return 2u; // UTF-16BE
+		case 12000: return 4u; // UTF-32LE
+		case 12001: return 4u; // UTF-32BE
+		default:    return 1u;
+	}
+}
+
+//! Fallback conversion that will at least work for ASCII characters
+void to_utf8_fallback(const std::string & from, std::string & to, codepage_id codepage) {
+	
+	size_t skip = get_encoding_size(codepage);
+	
+	size_t shift = 0;
+	switch(codepage) {
+		case  1201: shift = 1u * 8u; break; // UTF-16BE
+		case 12001: shift = 3u * 8u; break; // UTF-32BE
+		default: break;
+	}
+	
+	to.clear();
+	to.reserve(ceildiv(from.size(), skip));
+	
+	bool warn = false;
+	
+	for(std::string::const_iterator it = from.begin(); it != from.end();) {
+		
+		unicode_char unicode = 0;
+		for(size_t i = 0; i < skip; i++) {
+			unicode |= unicode_char(boost::uint8_t(*it++)) << (i * 8);
+		}
+		
+		char ascii = char((unicode >> shift) & 0x7f);
+		
+		// replace non-ASCII characters with underscores
+		if((unicode_char(ascii) << shift) != unicode) {
+			warn = true;
+			ascii = char(replacement_char);
+		}
+		
+		to.push_back(ascii);
+	}
+	
+	if(warn) {
+		log_warning << "Unknown data while converting from CP" << codepage << " to UTF-8.";
+	}
+	
+}
+
+bool is_utf8_continuation_byte(unicode_char chr) {
+	return (chr & 0xc0) == 0x80;
+}
+
+template <typename In>
+unicode_char utf8_read(In & it, In end, unicode_char replacement = replacement_char) {
+	
+	if(it == end) {
+		return unicode_char(-1);
+	}
+	unicode_char chr = boost::uint8_t(*it++);
+	
+	// For multi-byte characters, read the remaining bytes
+	if(chr & (1 << 7)) {
+		
+		if(is_utf8_continuation_byte(chr)) {
+			// Bad start position
+			return replacement;
+		}
+		
+		if(it == end || !is_utf8_continuation_byte(boost::uint8_t(*it))) {
+			// Unexpected end of multi-byte sequence
+			return replacement;
+		}
+		chr &= 0x3f, chr <<= 6, chr |= unicode_char(boost::uint8_t(*it++) & 0x3f);
+		
+		if(chr & (1 << (5 + 6))) {
+			
+			if(it == end || !is_utf8_continuation_byte(boost::uint8_t(*it))) {
+				// Unexpected end of multi-byte sequence
+				return replacement;
+			}
+			chr &= ~unicode_char(1 << (5 + 6)), chr <<= 6, chr |= unicode_char(boost::uint8_t(*it++) & 0x3f);
+			
+			if(chr & (1 << (4 + 6 + 6))) {
+				
+				if(it == end || !is_utf8_continuation_byte(boost::uint8_t(*it))) {
+					// Unexpected end of multi-byte sequence
+					return replacement;
+				}
+				chr &= ~unicode_char(1 << (4 + 6 + 6)), chr <<= 6, chr |= unicode_char(boost::uint8_t(*it++) & 0x3f);
+				
+				if(chr & (1 << (3 + 6 + 6 + 6))) {
+					// Illegal UTF-8 byte
+					return replacement;
+				}
+				
+			}
+		}
+	}
+	
+	return chr;
+}
+
+size_t utf8_length(unicode_char chr) {
+	if(chr < 0x80) {
+		return 1;
+	} else if(chr < 0x800) {
+		return 2;
+	} else if(chr < 0x10000) {
+		return 3;
+	} else if(chr <= 0x0010ffff) {
+		return 4;
+	}
+	return 1;
+}
+
+void utf8_write(std::string & to, unicode_char chr) {
+	
+	static const boost::uint8_t first_bytes[7] = {
+		0x00, 0x00, 0xc0, 0xe0, 0xf0, 0xf8, 0xfc
+	};
+	
+	// Get number of bytes to write
+	size_t length = utf8_length(chr);
+	
+	// Extract bytes to write
+	boost::uint8_t bytes[4];
+	switch(length) {
+		case 4: bytes[3] = static_cast<boost::uint8_t>((chr | 0x80) & 0xBF), chr >>= 6; /* fall-through */
+		case 3: bytes[2] = static_cast<boost::uint8_t>((chr | 0x80) & 0xBF), chr >>= 6; /* fall-through */
+		case 2: bytes[1] = static_cast<boost::uint8_t>((chr | 0x80) & 0xBF), chr >>= 6; /* fall-through */
+		case 1: bytes[0] = static_cast<boost::uint8_t>(chr | first_bytes[length]);
+		default: break;
+	}
+	
+	// Add them to the output
+	const boost::uint8_t * cur_byte = bytes;
+	switch(length) {
+		case 4: to.push_back(char(*cur_byte++)); /* fall-through */
+		case 3: to.push_back(char(*cur_byte++)); /* fall-through */
+		case 2: to.push_back(char(*cur_byte++)); /* fall-through */
+		case 1: to.push_back(char(*cur_byte++));
+		default: break;
+	}
+	
+}
+
+//! \return true c is is the first part of an UTF-16 surrogate pair
+bool is_utf16_high_surrogate(unicode_char chr) {
+	return chr >= 0xd800 && chr <= 0xdbff;
+}
+
+//! \return true c is is the second part of an UTF-16 surrogate pair
+bool is_utf16_low_surrogate(unicode_char chr) {
+	return chr >= 0xdc00 && chr <= 0xdfff;
+}
+
+void utf16le_to_utf8(const std::string & from, std::string & to) {
+	
+	if(from.size() % 2 != 0) {
+		log_warning << "Unexpected trailing byte in UTF-16 string.";
+	}
+	
+	to.clear();
+	to.reserve(from.size() / 2); // optimistically, most strings only have ASCII characters
+	
+	bool warn = false;
+	
+	std::string::const_iterator it = from.begin();
+	std::string::const_iterator end = from.end();
+	while(it != end) {
+		
+		unicode_char chr = boost::uint8_t(*it++);
+		if(it == end) {
+			warn = true;
+			utf8_write(to, replacement_char);
+			break;
+		}
+		chr |= unicode_char(boost::uint8_t(*it++)) << 8;
+		
+		// If it's a surrogate pair, convert to a single UTF-32 character
+		if(is_utf16_high_surrogate(chr)) {
+			if(it == end) {
+				warn = true;
+				utf8_write(to, replacement_char);
+				break;
+			}
+			unicode_char d = boost::uint8_t(*it++);
+			if(it == end) {
+				warn = true;
+				utf8_write(to, replacement_char);
+				break;
+			}
+			d |= unicode_char(boost::uint8_t(*it++)) << 8;
+			if(is_utf16_low_surrogate(d)) {
+				chr = ((chr - 0xd800) << 10) + (d - 0xdc00) + 0x0010000;
+			} else {
+				warn = true;
+				utf8_write(to, replacement_char);
+				continue;
+			}
+		}
+		
+		// Replace invalid characters
+		if(chr > 0x0010FFFF) {
+			warn = true;
+			// Invalid character (greater than the maximum unicode value)
+			utf8_write(to, replacement_char);
+			continue;
+		}
+		
+		utf8_write(to, chr);
+	}
+	
+	if(warn) {
+		log_warning << "Unexpected data while converting from UTF-16LE to UTF-8.";
+	}
+	
+}
+
+void utf8_to_utf16le(const std::string & from, std::string & to) {
+	
+	to.clear();
+	to.reserve(from.size() * 2); // optimistically, most strings only have ASCII characters
+	
+	bool warn = false;
+	
+	for(std::string::const_iterator i = from.begin(); i != from.end(); ) {
+		
+		unicode_char chr = utf8_read(i, from.end());
+		
+		if((chr >= 0xd800 && chr <= 0xdfff) || chr > 0x10ffff) {
+			chr = replacement_char;
+			warn = true;
+		} else if(chr >= 0x10000) {
+			chr -= 0x10000;
+			unicode_char high_surrogate = 0xd800 + (chr >> 10);
+			to.push_back(char(boost::uint8_t(high_surrogate)));
+			to.push_back(char(boost::uint8_t(high_surrogate >> 8)));
+			chr = 0xdc00 + (chr & 0x3ff);
+		}
+		
+		to.push_back(char(boost::uint8_t(chr)));
+		to.push_back(char(boost::uint8_t(chr >> 8)));
+	}
+	
+	if(warn) {
+		log_warning << "Unexpected data while converting from UTF-8 to UTF-16LE.";
+	}
+	
+}
+
+unicode_char windows1252_replacements[] = {
+	0x20ac, replacement_char, 0x201a, 0x192, 0x201e, 0x2026, 0x2020, 0x2021, 0x2c6,
+	0x2030, 0x160, 0x2039, 0x152, replacement_char, 0x17d, replacement_char,
+	replacement_char, 0x2018, 0x2019, 0x201c, 0x201d, 0x2022, 0x2013, 0x2014, 0x2dc,
+	0x2122, 0x161, 0x203a, 0x153, replacement_char, 0x17e, 0x178
+};
+
+BOOST_STATIC_ASSERT(sizeof(windows1252_replacements) == (160 - 128) * sizeof(*windows1252_replacements));
+
+void windows1252_to_utf8(const std::string & from, std::string & to) {
+	
+	to.clear();
+	to.reserve(from.size()); // optimistically, most strings only have ASCII characters
+	
+	bool warn = false;
+	
+	BOOST_FOREACH(char c, from) {
+		
+		// Windows-1252 maps almost directly to Unicode - yay!
+		unicode_char chr = boost::uint8_t(c);
+		if(chr >= 128 && chr < 160) {
+			chr = windows1252_replacements[chr - 128];
+			warn = warn || chr == replacement_char;
+		}
+		
+		utf8_write(to, chr);
+	}
+	
+	if(warn) {
+		log_warning << "Unexpected data while converting from Windows-1252 to UTF-8.";
+	}
+	
+}
+
+void utf8_to_windows1252(const std::string & from, std::string & to) {
+	
+	to.clear();
+	to.reserve(from.size()); // optimistically, most strings only have ASCII characters
+	
+	bool warn = false;
+	
+	for(std::string::const_iterator i = from.begin(); i != from.end(); ) {
+		
+		unicode_char chr = utf8_read(i, from.end());
+		
+		// Windows-1252 maps almost directly to Unicode - yay!
+		if(chr >= 256 || (chr >= 128 && chr < 160)) {
+			size_t i = 0;
+			for(; i < size_t(boost::size(windows1252_replacements)); i++) {
+				if(chr == windows1252_replacements[i] && windows1252_replacements[i] != replacement_char) {
+					break;
+				}
+			}
+			if(i < size_t(boost::size(windows1252_replacements))) {
+				chr = unicode_char(128 + i);
+			} else {
+				chr = replacement_char;
+				warn = true;
+			}
+		}
+		
+		to.push_back(char(boost::uint8_t(chr)));
+	}
+	
+	if(warn) {
+		log_warning << "Unsupported character while converting from UTF-8 to Windows-1252.";
+	}
+	
+}
+
+#if INNOEXTRACT_HAVE_ICONV
+
+typedef boost::unordered_map<codepage_id, iconv_t> converter_map;
+converter_map converters;
+
+iconv_t get_converter(codepage_id codepage) {
 	
 	// Try to reuse an existing converter if possible
 	converter_map::const_iterator i = converters.find(codepage);
@@ -422,9 +529,9 @@ static iconv_t get_converter(codepage_id codepage) {
 	return converters[codepage] = handle;
 }
 
-static bool to_utf8_iconv(const std::string & from, std::string & to, codepage_id cp) {
+bool to_utf8_iconv(const std::string & from, std::string & to, codepage_id codepage) {
 	
-	iconv_t converter = get_converter(cp);
+	iconv_t converter = get_converter(codepage);
 	if(converter == iconv_t(-1)) {
 		return false;
 	}
@@ -447,7 +554,7 @@ static bool to_utf8_iconv(const std::string & from, std::string & to, codepage_i
 	
 	iconv(converter, NULL, NULL, NULL, NULL);
 	
-	size_t skip = get_encoding_size(cp);
+	size_t skip = get_encoding_size(codepage);
 	
 	bool warn = false;
 	
@@ -465,9 +572,9 @@ static bool to_utf8_iconv(const std::string & from, std::string & to, codepage_i
 			} else if(/*errno == EILSEQ &&*/ insize >= 2) {
 				// invalid byte (sequence) - add a replacement char and try the next byte
 				if(outsize == 0) {
-					to.push_back(replacement_char);
+					to.push_back(char(replacement_char));
 				} else {
-					*outbuf = replacement_char;
+					*outbuf = char(replacement_char);
 					outsize--;
 				}
 				inbuf.buf += skip;
@@ -484,7 +591,7 @@ static bool to_utf8_iconv(const std::string & from, std::string & to, codepage_i
 	}
 	
 	if(warn) {
-		log_warning << "Unexpected data while converting from CP" << cp << " to UTF-8.";
+		log_warning << "Unexpected data while converting from CP" << codepage << " to UTF-8.";
 	}
 	
 	to.resize(outbase);
@@ -496,7 +603,7 @@ static bool to_utf8_iconv(const std::string & from, std::string & to, codepage_i
 
 #if INNOEXTRACT_HAVE_WIN32_CONV
 
-static std::string windows_error_string(DWORD code) {
+std::string windows_error_string(DWORD code) {
 	char * error;
 	DWORD n = FormatMessageA(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_ALLOCATE_BUFFER,
 	                         NULL, code, 0, reinterpret_cast<char *>(&error), 0,
@@ -513,7 +620,7 @@ static std::string windows_error_string(DWORD code) {
 	}
 }
 
-static bool to_utf8_win32(const std::string & from, std::string & to, codepage_id cp) {
+bool to_utf8_win32(const std::string & from, std::string & to, codepage_id codepage) {
 	
 	int ret = 0;
 	
@@ -521,18 +628,18 @@ static bool to_utf8_win32(const std::string & from, std::string & to, codepage_i
 	const WCHAR * utf16;
 	int utf16_size;
 	std::vector<WCHAR> buffer;
-	if(cp == cp_utf16le) {
+	if(codepage == cp_utf16le) {
 		utf16 = reinterpret_cast<const WCHAR *>(from.data());
 		utf16_size = int(from.size()) / 2;
 	} else {
-		utf16_size = MultiByteToWideChar(cp, 0, from.data(), int(from.length()), NULL, 0);
+		utf16_size = MultiByteToWideChar(codepage, 0, from.data(), int(from.length()), NULL, 0);
 		if(utf16_size > 0) {
 			buffer.resize(size_t(utf16_size));
-			ret = MultiByteToWideChar(cp, 0, from.data(), int(from.length()),
+			ret = MultiByteToWideChar(codepage, 0, from.data(), int(from.length()),
 			                          &buffer.front(), utf16_size);
 		}
 		if(utf16_size <= 0 || ret <= 0) {
-			log_warning << "Error while converting from CP" << cp << " to UTF-16: "
+			log_warning << "Error while converting from CP" << codepage << " to UTF-16: "
 			            << windows_error_string(GetLastError());
 			return false;
 		}
@@ -559,38 +666,76 @@ static bool to_utf8_win32(const std::string & from, std::string & to, codepage_i
 
 } // anonymous namespace
 
-void to_utf8(const std::string & from, std::string & to, codepage_id cp) {
+void to_utf8(const std::string & from, std::string & to, codepage_id codepage) {
 	
 	if(from.empty()) {
 		to.clear();
 		return;
 	}
 	
-	if(cp == cp_utf8 || cp == cp_ascii) {
+	if(codepage == cp_utf8 || codepage == cp_ascii) {
 		to = from;
 		return;
 	}
 	
-	#if INNOEXTRACT_HAVE_BUILTIN_CONV
-	if(to_utf8_builtin(from, to, cp)) {
-		return;
+	switch(codepage) {
+		case cp_utf16le:     utf16le_to_utf8(from, to); return;
+		case cp_windows1252: windows1252_to_utf8(from, to); return;
+		case cp_iso_8859_1:  windows1252_to_utf8(from, to); return;
+		default: break;
 	}
-	#endif
 	
 	#if INNOEXTRACT_HAVE_ICONV
-	if(to_utf8_iconv(from, to, cp)) {
+	if(to_utf8_iconv(from, to, codepage)) {
 		return;
 	}
 	#endif
 	
 	#if INNOEXTRACT_HAVE_WIN32_CONV
-	if(to_utf8_win32(from, to, cp)) {
+	if(to_utf8_win32(from, to, codepage)) {
 		return;
 	}
 	#endif
 	
-	to_utf8_fallback(from, to, cp);
+	to_utf8_fallback(from, to, codepage);
 	
 }
 
+void from_utf8(const std::string & from, std::string & to, codepage_id codepage) {
+	
+	if(from.empty()) {
+		to.clear();
+		return;
+	}
+	
+	if(codepage == cp_utf8) {
+		to = from;
+		return;
+	}
+	
+	switch(codepage) {
+		case cp_utf16le:     utf8_to_utf16le(from, to); return;
+		case cp_windows1252: utf8_to_windows1252(from, to); return;
+		case cp_iso_8859_1:  utf8_to_windows1252(from, to); return;
+		default: {
+			log_warning << "Unsupported output codepage: " << codepage;
+			to = from;
+		}
+	}
+	
+}
+
+std::string encoding_name(codepage_id codepage) {
+	
+	const char * name = get_encoding_name(codepage);
+	if(name) {
+		return name;
+	}
+	
+	std::ostringstream oss;
+	oss << "Windows-" << codepage;
+	
+	return oss.str();
+}
+
 } // namespace util
